from datetime import datetime
from typing import List, Optional, Tuple

import pandas

from spectrominer.parser.analysis import Analysis
from spectrominer.parser.molecule_results import MResult, MoleculeResults
from spectrominer.parser.util import decode_date


class Parser:

    def __init__(self, filepath: str, delimiter: str = ',', quotechar: str = '"') -> None:
        self.df = pandas.read_csv(
            filepath,
            delimiter=delimiter,
            quotechar=quotechar,
            header=[0, 1]
        )

    def get_analyzes(self) -> List[Analysis]:
        self._remove_empty_columns()
        self._fill_column_names()

        analysis_list = []

        for index, row in self.df.iterrows():
            molecules_results = self._get_molecules_results(index)

            analysis_list.append(
                Analysis(
                    index=index,
                    name=self._find_analysis_name(row),
                    date=self._find_date(row),
                    results=molecules_results
                )
            )

        analysis_list.sort()

        return analysis_list

    def get_molecule_names(self) -> List[str]:
        # Multiple label and multiple molecules: set('Lactic acid M+1, Lactic acid M+2, ..., Malic acid M+0, ...)
        molecules_labels = set(column[0] for column in self.df.columns if 'Results' in column[0])
        # Multiple label per molecule set('Lactic acid M+1, Lactic acid M+2, ..., Malic acid M+0, ...)
        molecule_names = set(label.split('M+')[0].strip() for label in molecules_labels if 'M+' in label)

        return list(molecule_names)

    def get_analyzes_of_given_molecule(self, molecule_name: str) -> List[Analysis]:
        analyzes_of_given_molecules = self.get_analyzes()

        for analysis in analyzes_of_given_molecules:
            analysis.results = [r for r in analysis.results if r.name == molecule_name]

        return analyzes_of_given_molecules

    def _get_molecules_results(self, row_index: int) -> List[MoleculeResults]:
        molecules_results = []

        molecule_names = self.get_molecule_names()

        # Multiple label and multiple molecules: set('Lactic acid M+1, Lactic acid M+2, ..., Malic acid M+0, ...)
        molecules_labels = set(column[0] for column in self.df.columns if 'Results' in column[0])

        for name in molecule_names:
            # Getting all labels with the molecule name
            molecule_labels = [label for label in molecules_labels if name in label]
            # Removing results label without the "M+"
            molecule_labels = [label for label in molecule_labels if 'M+' in label]

            if not molecule_labels:
                continue

            m_results = []

            for label in molecule_labels:
                m_number = int(label.split('M+')[1].split(' ')[0])  # Example of label: "L-Pyroglutamic M+2 Results"

                m_results.append(
                    MResult(
                        m_number=m_number,
                        retention_time=self._find_retention_time(label, row_index),
                        area=self._find_area(label, row_index),
                        istd_resp_ratio=float(self.df[(label, 'ISTD Resp. Ratio')].iloc[[row_index]]),
                    )
                )

            m_results.sort()
            molecules_results.append(MoleculeResults(name=name, m_results=m_results))

        molecules_results.sort()

        return molecules_results

    def _find_analysis_name(self, row: pandas.Series) -> str:
        for column in row.index:
            c1, c2 = column  # Column are tuple (2 lines/headers columns)

            if c2.lower() == 'name':
                return row[column]

        raise ValueError('Column of the analyzes names not found')

    def _find_date(self, row: pandas.Series) -> Optional[datetime]:
        for column in row.index:
            c1, c2 = column  # Column are tuple (2 lines/headers columns)

            if 'date' in c2.lower():
                return decode_date(row[column])

        return None

    def _find_retention_time(self, label: str, row_index: int) -> Optional[float]:
        try:
            float(self.df[(label, 'RT')].iloc[[row_index]])
        except KeyError:
            return None

    def _find_area(self, label: str, row_index: int) -> Optional[float]:
        try:
            float(self.df[(label, 'Area')].iloc[[row_index]])
        except KeyError:
            return None

    def _fill_column_names(self) -> None:
        """
        Extend the column name to all following column without name

        --------------       -----------
        name | no_name       name | name
        RT   | Area     -->  RT   | Area
        --------------       -----------
        """
        new_columns = []
        last_column_name = self.df.columns[0][0]

        for column in self.df.columns:
            if 'Unnamed:' in column[0]:
                column = (last_column_name, column[1])
            else:
                last_column_name = column[0]

            new_columns.append(column)

        self.df.columns = new_columns

    def _remove_empty_columns(self) -> None:
        columns_to_remove = []

        for column in self.df.columns:
            if 'Unnamed:' in column[0] and 'Unnamed:' in column[1]:
                columns_to_remove.append(column)

        for column in columns_to_remove:
            del self.df[column]



import tkinter
from tkinter import ttk
from tkinter.scrolledtext import ScrolledText


class PopUp(tkinter.Toplevel):

    def __init__(self, message: str):
        super(PopUp, self).__init__()
        self.geometry('800x600')

        self.wm_title('Message')

        btn = ttk.Button(self, text='Ok', command=self.destroy)
        btn.pack(anchor='e', side=tkinter.BOTTOM, padx=10, pady=10)

        text_box = ScrolledText(
            self,
            width=360,
            height=230,
            wrap=tkinter.WORD
        )
        text_box.pack(padx=10, pady=10, fill=tkinter.BOTH, expand=True)
        text_box.insert(tkinter.INSERT, message)
        text_box.config(state='disabled')



import tkinter
from tkinter import filedialog, ttk
from typing import List, Optional

import matplotlib.pyplot as plt

from spectrominer.corrections.experimental import apply_experimental_corrections
from spectrominer.parser import Parser
from spectrominer.parser.analysis import Analysis
from spectrominer.ui.correction_wizard import CorrectionWizard
from spectrominer.ui.file_selector_frame import FileSelectorFrame


class MainFrame(ttk.Frame):

    def __init__(self, root: tkinter.Tk, **kw):
        super().__init__(root, **kw)
        self.root = root

        self.parser: Optional[Parser] = None
        self.experimental_correction_applied: bool = False
        self.control_analyzes_indexes: List[int] = []
        self.theoretical_correction_applied: False = False
        self.relative_result = tkinter.IntVar()

        # Widgets
        self.cb_molecule = ttk.Combobox(self.root, values=[], state='readonly')
        self.cb_molecule.bind('<<ComboboxSelected>>', self.recalculate_results)
        self.btn_apply_experimental_corrections = ttk.Button(
            self.root,
            text='Apply corrections with control measures',
            command=self._apply_experimental_corrections
        )
        self.table = ttk.Treeview(self.root)
        self.scrollbar = ttk.Scrollbar()
        self._set_table()
        self.cb_m_value = ttk.Combobox(self.root, values=[], state='readonly')
        self.btn_show_histogram = ttk.Button(self.root, text='Show histogram', command=self._show_histogram)
        self.btn_export = ttk.Button(
            self.root,
            text='Export metabolite data',
            command=lambda: self._export_data(all_data=False)
        )
        self.btn_export_all = ttk.Button(
            self.root,
            text='Export all data',
            command=lambda: self._export_data(all_data=True)
        )

        # Frames
        FileSelectorFrame(self, width=760, height=90).place(x=320, y=20)

        # Layout
        ttk.Label(self.root, text='Metabolite:').place(x=320, y=130)
        self.cb_molecule.place(x=400, y=125, width=200, height=25)
        ttk.Checkbutton(
            self.root,
            text='Relative results',
            variable=self.relative_result,
            command=self.recalculate_results,
        ).place(x=630, y=125, width=130, height=25)
        self.btn_apply_experimental_corrections.place(x=780, y=125, width=300, height=25)
        self.cb_m_value.place(x=50, y=840, width=115, height=30)
        self.btn_show_histogram.place(x=200, y=840, width=115, height=30)
        self.btn_export.place(x=1050, y=840, width=180, height=30)
        self.btn_export_all.place(x=1250, y=840, width=115, height=30)

    def _set_table(self):
        del self.scrollbar
        self.scrollbar = ttk.Scrollbar(self.root)
        self.scrollbar.place(x=1350, y=160, width=20, height=660)
        self.table.place(x=50, y=160, width=1300, height=660)

        self.scrollbar.config(command=self.table.yview)
        self.table.config(yscrollcommand=self.scrollbar.set)

        self.table.bind("<Delete>", self._delete_row)

    def recalculate_results(self, *_):
        analyzes = self._get_analyzes()
        nbr_of_M = len(analyzes[0].results[0].m_results)

        # Setting columns
        del self.table
        self.table = ttk.Treeview(self.root, columns=list(range(nbr_of_M + 1)), show='headings')
        self._set_table()

        self.table.heading(0, text='Analysis')
        self.table.column(0, minwidth=80)
        for i in range(nbr_of_M):
            self.table.heading(i + 1, text=f'M+{i}')
            self.table.column(i + 1, anchor='center', width=50)

        # Setting rows
        for analysis in analyzes:
            values = [analysis.name] + [f'{r.istd_resp_ratio:e}' for r in analysis.results[0].m_results]
            self.table.insert('', 'end', values=values)

        # Showing available M values that can be plot
        self.cb_m_value.config(values=[f'M+{i}' for i in range(nbr_of_M)])
        self.cb_m_value.current(0)

    def _apply_experimental_corrections(self):
        if not self.experimental_correction_applied and self.cb_molecule.get() != '':
            self.wait_window(CorrectionWizard(self))

        else:
            self.experimental_correction_applied = False
            self.btn_apply_experimental_corrections.config(text='Apply corrections with control measures')
            self.recalculate_results()

    def _export_data(self, all_data: bool):
        with filedialog.asksaveasfile(mode='w', title='Select file', defaultextension='.csv') as file:
            analyzes = self._get_analyzes(all_data=all_data)
            header: List[str] = ['Analysis']
            rows: List[List[str]] = [[analysis.name] for analysis in analyzes]

            for molecule_result in analyzes[0].results:
                header += [f'{molecule_result.name} M+{m_result.m_number}' for m_result in molecule_result.m_results]

            for i, analysis in enumerate(analyzes):
                for molecule_result in analysis.results:
                    rows[i] += [str(r.istd_resp_ratio) for r in molecule_result.m_results]

            file.write(','.join(header) + '\n')
            for row in rows:
                file.write(','.join(row) + '\n')

    def _get_analyzes(self, all_data: bool = False) -> List[Analysis]:
        if all_data:
            analyzes = self.parser.get_analyzes()
        else:
            analyzes = self.parser.get_analyzes_of_given_molecule(self.cb_molecule.get())

        if self.experimental_correction_applied:
            control_analyzes = self._find_control_analyzes(analyzes)
            with_relative_results = bool(self.relative_result.get())
            analyzes = apply_experimental_corrections(analyzes, control_analyzes, with_relative_results)

        elif self.theoretical_correction_applied:
            raise NotImplementedError('Theoretical corrections have not been implemented')

        return analyzes

    def _find_control_analyzes(self, analyzes: List[Analysis]) -> List[Analysis]:
        control_analyzes = []

        for analysis_index in self.control_analyzes_indexes:
            for analysis in analyzes:
                if analysis.index == analysis_index:
                    control_analyzes.append(analysis)
                    break

        return control_analyzes

    def _show_histogram(self):
        analyzes = self._get_analyzes()
        molecule_name = f'{analyzes[0].results[0].name} {self.cb_m_value.get()}'
        analysis_names, analysis_results = [], []

        for analysis in analyzes:
            for m_result in analysis.results[0].m_results:
                if f'M+{m_result.m_number}' == self.cb_m_value.get():
                    analysis_names.append(analysis.name)
                    analysis_results.append(m_result.istd_resp_ratio)
                    continue

        fig = plt.figure(figsize=(12, 8))
        ax = fig.add_subplot(111)
        ax.bar(analysis_names, analysis_results)
        ax.set_xticklabels(analysis_names, rotation=45, ha='right')

        plt.title(molecule_name)
        plt.tight_layout()
        plt.show()

    def _delete_row(self, *_):
        for row in self.table.selection():
            self.table.delete(row)



import tkinter
from tkinter import ttk
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from spectrominer.ui.main_frame import MainFrame


class CorrectionWizard(tkinter.Toplevel):

    def __init__(self, parent: 'MainFrame'):
        super().__init__()
        self.parent = parent

        self.geometry('800x600')
        self.wm_title('Correction wizard')

        ttk.Label(self, text='Select control measures').place(x=20, y=20)
        self.table = ttk.Treeview(self, columns=[0], show='headings')
        self.table.heading(0, text='Analysis')
        self.scrollbar = ttk.Scrollbar()
        self.set_table()

        btn_ok = ttk.Button(self, text='Ok', command=self.correction_done)
        btn_ok.place(x=680, y=540, width=100, height=30)
        ttk.Checkbutton(self, text='Relative results', variable=self.parent.relative_result).place(x=540, y=540, width=125, height=30)

    def correction_done(self):
        self.parent.control_analyzes_indexes = []

        for row in self.table.selection():
            control_analysis_name = self.table.item(row)['values'][0]

            for analysis in self.parent.parser.get_analyzes():
                if analysis.name == control_analysis_name:
                    self.parent.control_analyzes_indexes.append(analysis.index)
                    break

        # If nothing have been selected
        if not self.parent.control_analyzes_indexes:
            return

        self.parent.experimental_correction_applied = True
        self.parent.btn_apply_experimental_corrections.config(text='Remove corrections with control measures')
        self.parent.recalculate_results()

        self.destroy()

    def set_table(self):
        del self.scrollbar
        self.scrollbar = ttk.Scrollbar(self)
        self.scrollbar.place(x=760, y=40, width=20, height=480)
        self.table.place(x=20, y=40, width=740, height=480)

        for analysis in self.parent.parser.get_analyzes():
            self.table.insert('', 'end', values=[analysis.name])

        self.scrollbar.config(command=self.table.yview)
        self.table.config(yscrollcommand=self.scrollbar.set)



from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from spectrominer.parser.molecule_results import MoleculeResults


@dataclass
class Analysis:
    index: int
    name: str
    date: Optional[datetime]

    results: List[MoleculeResults]

    def __lt__(self, other: 'Analysis') -> bool:
        return self.name < other.name

    def __getitem__(self, molecule_name: str) -> MoleculeResults:
        for molecule_result in self.results:
            if molecule_result.name == molecule_name:
                return molecule_result

        raise KeyError(molecule_name)



from dataclasses import dataclass
from typing import List, Optional


@dataclass
class MResult:
    m_number: int

    retention_time: Optional[float]
    area: Optional[float]
    istd_resp_ratio: float

    def __lt__(self, other: 'MResult') -> bool:
        return self.m_number < other.m_number


@dataclass
class MoleculeResults:
    name: str

    m_results: List[MResult]

    def __lt__(self, other: 'MoleculeResults') -> bool:
        return self.name < other.name

    def __getitem__(self, m_number: int) -> MResult:
        for m_result in self.m_results:
            if m_result.m_number == m_number:
                return m_result

        raise KeyError(f'M_number {m_number} not found')



import copy
from typing import List

import pandas

from spectrominer.parser.analysis import Analysis
from spectrominer.parser.molecule_results import MoleculeResults


def normalize_analyzes(analyzes: List[Analysis]) -> List[Analysis]:
    analyzes = copy.deepcopy(analyzes)

    for analysis in analyzes:
        for molecule_results in analysis.results:
            sum_of_results = sum_m_results(molecule_results)

            for m_result in molecule_results.m_results:
                m_result.istd_resp_ratio /= sum_of_results

    return analyzes


def sum_m_results(molecule_results: MoleculeResults) -> float:
    sum_of_results = 0

    for m_result in molecule_results.m_results:
        if not pandas.isna(m_result.istd_resp_ratio):
            sum_of_results += m_result.istd_resp_ratio

    return sum_of_results



import tkinter
import traceback
from tkinter import filedialog, ttk

from spectrominer.parser import Parser
from spectrominer.ui.popup import PopUp


class FileSelectorFrame(ttk.LabelFrame):

    def __init__(self, parent: 'MainFrame', **kw):
        super().__init__(parent.master, text='Select file to parse', **kw)
        self.parent = parent

        # Widgets
        self.inline_filepath = ttk.Entry(self)
        self.combobox_delimiter = ttk.Combobox(self, values=[',', ';', ' '], state='readonly')
        self.combobox_delimiter.current(0)
        self.btn_browse = ttk.Button(self, text='Browse', command=self.browse_file)

        # Layout
        self.inline_filepath.place(x=20, y=25, width=500, height=30)
        ttk.Separator(self, orient='vertical').place(x=530, y=5, height=55)
        ttk.Label(self, text='Delimiter').place(x=540, y=5)
        self.combobox_delimiter.place(x=540, y=25, width=80, height=30)
        ttk.Separator(self, orient='vertical').place(x=630, y=5, height=55)
        self.btn_browse.place(x=640, y=25, width=100, height=30)

    def browse_file(self):
        filepath = filedialog.askopenfilename(title='Browse')
        if filepath == ():  # No file have been selected
            return

        try:
            self.parent.parser = Parser(filepath, delimiter=self.combobox_delimiter.get())
            self.inline_filepath.delete(0, tkinter.END)
            self.inline_filepath.insert(0, filepath)

            molecule_names = self.parent.parser.get_molecule_names()
            molecule_names.sort()

            self.parent.cb_molecule.config(values=molecule_names)
            self.parent.cb_molecule.current(0)
            self.parent.recalculate_results()
        except Exception:
            PopUp(message=traceback.format_exc())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['spectrominer',
 'spectrominer.corrections',
 'spectrominer.parser',
 'spectrominer.ui']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.4.1,<4.0.0', 'pandas>=1.2.3,<2.0.0', 'pyinstaller>=4.3,<5.0']

entry_points = \
{'console_scripts': ['spectrominer = spectrominer.ui.app:start']}

setup_kwargs = {
    'name': 'spectrominer',
    'version': '0.0.1',
    'description': 'Spectrominer is a software that allows you to apply corrections to mass spectrometry data.',
    'long_description': '# SpectroMiner\nSpectrominer is a software that allows you to apply corrections to mass spectrometry data.\nMore precisely, it uses control measures in order to subtract the natural presence of isotopes.\n\n\n## Installation\n### Windows\nsee the [releases](https://github.com/ulaval-rs/spectrominer/releases)\n\nTo run spectrominer, run the .exe.\n\n### Macos\nsee the [releases](https://github.com/ulaval-rs/spectrominer/releases)\n\n### PyPi\nTo install:\n```bash\n$ pip install spectrominer\n```\nTo run spectrominer:\n```bash\n$ spectrominer\n```\nor\n```bash\n$ /path/to/python/bin/spectrominer\n```\n',
    'author': 'Gabriel Couture',
    'author_email': 'gacou54@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ulaval-rs/spectrominer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)



import tkinter

from spectrominer.ui.main_frame import MainFrame


class App(tkinter.Tk):

    def __init__(self):
        super().__init__()

        self.geometry('1400x900')
        self.winfo_toplevel().title('Spectrominer')  # Title
        MainFrame(self)


def start():
    app = App()
    app.mainloop()


if __name__ == '__main__':
    start()



import copy
from typing import Dict, List

import numpy

from spectrominer.corrections.util import normalize_analyzes
from spectrominer.parser.analysis import Analysis


def apply_experimental_corrections(
        analyzes: List[Analysis],
        control_analyzes: List[Analysis],
        with_relative_results: bool) -> List[Analysis]:
    original_analyzes = copy.deepcopy(analyzes)
    analyzes = _remove_control_analyzes_from_analyzes(analyzes, control_analyzes)

    # Normalization
    normalized_analyzes = normalize_analyzes(analyzes)

    normalized_control_analyzes = normalize_analyzes(control_analyzes)
    normalized_averaged_control_analysis = _calculate_average_control_analysis(normalized_control_analyzes)

    # Getting the proportion of M+* values (relative results)
    analyzes = _remove_relative_abundance(normalized_analyzes, normalized_averaged_control_analysis)

    # Getting the absolute values of the presence of M+* results
    if with_relative_results:
        return _remove_negative_values(analyzes)

    return _convert_results_in_absolute(analyzes, normalized_analyzes, original_analyzes)


def _remove_control_analyzes_from_analyzes(
        analyzes: List[Analysis],
        control_analyzes: List[Analysis]) -> List[Analysis]:
    for control_analysis in control_analyzes:
        analyzes.remove(control_analysis)

    return analyzes


def _calculate_average_control_analysis(control_analyzes: List[Analysis]) -> Analysis:
    average_control_analysis = control_analyzes[0]
    average_control_analysis.name = 'Average control analysis'

    number_of_values_per_m_result_per_molecule = _find_number_of_values_per_m_result_per_molecule(control_analyzes)

    # Sum on all the values
    for i, analysis in enumerate(control_analyzes):
        # The first analysis will be the average.
        if i == 0:
            continue

        for molecule_results in analysis.results:
            for m_result in molecule_results.m_results:
                # If value is NaN, no need to add it
                if numpy.isnan(m_result.istd_resp_ratio):
                    continue

                if numpy.isnan(average_control_analysis[molecule_results.name][m_result.m_number].istd_resp_ratio):
                    average_control_analysis[molecule_results.name][
                        m_result.m_number].istd_resp_ratio = m_result.istd_resp_ratio
                else:
                    average_control_analysis[molecule_results.name][
                        m_result.m_number].istd_resp_ratio += m_result.istd_resp_ratio

    # Dividing by the number of analyzes to get the average
    for molecule_results in average_control_analysis.results:
        for m_result in molecule_results.m_results:
            # These values are useless
            m_result.retention_time = numpy.NaN
            m_result.area = numpy.NaN

            if number_of_values_per_m_result_per_molecule[molecule_results.name][m_result.m_number] == 0:
                raise ValueError(f'{molecule_results.name} M+{m_result.m_number} has no control values')

            m_result.istd_resp_ratio /= number_of_values_per_m_result_per_molecule[
                molecule_results.name][m_result.m_number]

    return average_control_analysis


def _find_number_of_values_per_m_result_per_molecule(analyzes: List[Analysis]) -> Dict[str, Dict[int, int]]:
    number_of_values_per_m_result_per_molecule: Dict[str, Dict[int, int]] = {}

    for analysis in analyzes:
        for molecule_results in analysis.results:
            number_of_values_per_m_result_per_molecule[molecule_results.name] = {
                m_result.m_number: 0 for m_result in molecule_results.m_results
            }

    for analysis in analyzes:
        for molecule_results in analysis.results:
            for m_result in molecule_results.m_results:
                if not numpy.isnan(m_result.istd_resp_ratio):
                    number_of_values_per_m_result_per_molecule[molecule_results.name][m_result.m_number] += 1

    return number_of_values_per_m_result_per_molecule


def _remove_relative_abundance(
        normalized_analyzes: List[Analysis],
        normalized_averaged_control_analysis: Analysis) -> List[Analysis]:
    analyzes = copy.deepcopy(normalized_analyzes)

    for analysis in analyzes:
        for molecule_result, control_molecule_results in zip(analysis.results,
                                                             normalized_averaged_control_analysis.results):
            assert molecule_result.name == control_molecule_results.name, 'Molecule should be the same, since molecules are sorted by name'

            for m_result, control_m_result in zip(molecule_result.m_results, control_molecule_results.m_results):
                assert m_result.m_number == control_m_result.m_number, 'M+ number should be the same, since they are sorted'

                # When M+0, remove the natural fraction of C13 (all other M+*)
                if m_result.m_number == 0:
                    m_result.istd_resp_ratio = m_result.istd_resp_ratio - sum(
                        m.istd_resp_ratio for m in control_molecule_results.m_results
                    ) / len(control_molecule_results.m_results)

                else:
                    m_result.istd_resp_ratio -= control_m_result.istd_resp_ratio

    return analyzes


def _convert_results_in_absolute(
        relative_analyzes: List[Analysis],
        normalized_analyzes: List[Analysis],
        original_analyzes: List[Analysis]) -> List[Analysis]:
    analyzes = relative_analyzes

    for analysis, normalized_analysis, original_analysis in zip(analyzes, normalized_analyzes, original_analyzes):
        for molecule_result, normalized_molecule_result, original_molecule_result in zip(analysis.results,
                                                                                         normalized_analysis.results,
                                                                                         original_analysis.results):
            for m_result, normalized_m_result, original_m_result in zip(molecule_result.m_results,
                                                                        normalized_molecule_result.m_results,
                                                                        original_molecule_result.m_results):
                if m_result.m_number == 0:
                    m_result.istd_resp_ratio = original_m_result.istd_resp_ratio

                elif m_result.istd_resp_ratio < 0:
                    m_result.istd_resp_ratio = 0

                else:
                    m_result.istd_resp_ratio = m_result.istd_resp_ratio * original_m_result.istd_resp_ratio / normalized_m_result.istd_resp_ratio

    return analyzes


def _remove_negative_values(analyzes: List[Analysis]) -> List[Analysis]:
    # TODO: I'm not sure this is pertinent
    for analysis in analyzes:
        for molecule_result in analysis.results:
            for m_result in molecule_result.m_results:
                if m_result.istd_resp_ratio < 0:
                    m_result.istd_resp_ratio = 0

    return analyzes
